<?php
if ( ! defined( 'ABSPATH' ) ) {
    exit( 'restricted access' );
}

if ( ! function_exists( 'facturozor_recursive_sanitize_text_field' ) ) {
  function facturozor_recursive_sanitize_text_field( $array ) {
    foreach ( $array as $key => &$value ) {
        if ( is_array( $value ) ) {
            $value = facturozor_recursive_sanitize_text_field( $value );
        } else {
            $value = sanitize_text_field( $value );
        }
    }
    return $array;
  }
}

/*
 * This is a function that creates admin menu.
 */
if ( ! function_exists( 'wc_facturozor_menu' ) ) {
    add_action( 'admin_menu', 'wc_facturozor_menu' );
    function wc_facturozor_menu() {
        
      add_menu_page( 'Facturozor', 'WC - Facturozor', 'manage_options', 'wc_facturozor_sync', 'wc_facturozor_sync_fn', 'dashicons-migrate' );
      add_submenu_page( 'wc_facturozor_sync', __( 'Facturozor Fields Settings' ), __( 'Fields Settings' ), 'manage_options', 'wc_facturozor_fields_settings', 'wc_facturozor_fields_settings_fn' );
      add_submenu_page( 'wc_facturozor_sync', __( 'Facturozor Configuration' ), __( 'Settings' ), 'manage_options', 'wc_facturozor_settings', 'wc_facturozor_settings_fn' );
        
    }
}

/*
 * This is a function for activating plugin.
 */
if ( ! function_exists( 'wc_facturozor_settings_fn' ) ) {
  function wc_facturozor_settings_fn() {
      
    if ( isset( $_REQUEST['submit'] ) ) {
      $wc_facturozor_api_key = sanitize_text_field($_REQUEST['wc_facturozor_api_key']);
      if ( $wc_facturozor_api_key ) {
        $dataObj = array(
          'api_key' => $wc_facturozor_api_key,
          'site_url'        => site_url(),
          'status'        => 'verify',
        );
        $options = [
          'body'        => wp_json_encode( $dataObj ),
          'headers'     => [
              'Content-Type' => 'application/json',
          ],          
        ];
        $url = "https://www.facturozor.app/v1/api/verifyWCKey";
        $apiResponse = wp_remote_post( $url, $options );
        $response = json_decode( wp_remote_retrieve_body( $apiResponse ) );        
        if ( isset( $response->status )  && $response->status == 'success' ) {
          update_site_option( 'wc_facturozor_active', 1 );
          update_option( 'wc_facturozor_api_key', $wc_facturozor_api_key );
          $redirect_uri = menu_page_url( 'wc_facturozor_fields_settings', 0 );
          //$redirect_uri =  esc_attr($redirect_uri);
          ?>
            <div class="notice notice-success is-dismissible">
                <p><?php _e( 'Connection done. Please do not close.' ); ?></p>
            </div>
            <script type="text/javascript">
              jQuery( document ).ready( function( $ ) {
                window.setTimeout(function(){
                    window.location.replace( '<?php echo esc_attr($redirect_uri); ?>' );
                }, 3000);
              });
            </script>
          <?php
        }else{
          $log = "Error auth : verifyWCKey\n";
          $log .= "Msg: ".$response->msg."\n";
          $log .= "Date: ".date( 'Y-m-d H:i:s' )."\n\n"; 
          update_site_option( 'wc_facturozor_active', 0);
          update_option( 'wc_facturozor_api_key', "" );
          ?>
            <div class="notice notice-error is-dismissible">
            <p><?php _e( $log ); ?></p>
              <p><?php _e( 'Connection not validated. Please try again' ); ?></p>
            </div>
          <?php
        }
      }
    }         
    $wc_facturozor_api_key = get_option( 'wc_facturozor_api_key' );      
    $wc_facturozor_active = get_site_option('wc_facturozor_active');  
    ?>
      <div class="wrap">                
        <h1><?php _e( 'Facturozor Settings' ); ?></h1>
        <hr>
        <form method="post">
          <table class="form-table">
            <tbody>
              <tr>
                <th scope="row"><label><?php _e( 'API Secret' ); ?> <span class="description">(*)</span></label></th>
                <td>
                  <input class="regular-text" type="text" name="wc_facturozor_api_key" value="<?php echo esc_attr($wc_acturozor_api_key); ?>" required />
                </td>
              </tr>
            </tbody>
          </table>
          <p><input type='submit' class='button-primary' name="submit" value="<?php _e( 'Authorize' ); ?>" /></p>
        </form>
        <?php 
        if(intval($wc_facturozor_active) == 1){
          ?>
          <div class="notice notice-success">
              <p><?php _e( 'Connection validated.' ); ?></p>
          </div>
          <?php
        }
        ?>
        <?php 
        if(intval($wc_facturozor_active) == 0){
          ?>
          <div class="notice notice-warning">
              <p><?php _e( 'Connection not validated.' ); ?></p>
          </div>
          <?php 
        }
        ?>
      </div>
    <?php
  }
}

/*
 * Settings.
 a:2:{s:20:"wc_facturozor_fields";a:15:{s:10:"first_name";a:2:{s:3:"key";s:10:"first_name";s:4:"type";s:4:"text";}s:9:"last_name";a:2:{s:3:"key";s:9:"last_name";s:4:"type";s:4:"text";}s:10:"user_email";a:2:{s:3:"key";s:5:"email";s:4:"type";s:4:"text";}s:15:"billing_company";a:2:{s:3:"key";s:18:"name_of_enterprise";s:4:"type";s:4:"text";}s:17:"billing_address_1";a:2:{s:3:"key";s:7:"address";s:4:"type";s:4:"text";}s:12:"billing_city";a:2:{s:3:"key";s:4:"city";s:4:"type";s:4:"text";}s:13:"billing_state";a:2:{s:3:"key";s:5:"state";s:4:"type";s:4:"text";}s:16:"billing_postcode";a:2:{s:3:"key";s:3:"zip";s:4:"type";s:4:"text";}s:13:"billing_phone";a:2:{s:3:"key";s:5:"phone";s:4:"type";s:4:"text";}s:18:"shipping_address_1";a:2:{s:3:"key";s:16:"delivery_address";s:4:"type";s:4:"text";}s:13:"shipping_city";a:2:{s:3:"key";s:13:"delivery_city";s:4:"type";s:4:"text";}s:17:"shipping_postcode";a:2:{s:3:"key";s:12:"delivery_zip";s:4:"type";s:4:"text";}s:14:"shipping_state";a:2:{s:3:"key";s:14:"delivery_state";s:4:"type";s:4:"text";}s:11:"description";a:2:{s:3:"key";s:4:"note";s:4:"type";s:4:"text";}s:12:"display_name";a:2:{s:3:"key";s:4:"name";s:4:"type";s:4:"text";}}s:12:"module_saved";i:1;}
 */
if ( ! function_exists( 'wc_facturozor_fields_settings_fn' ) ) {
  function wc_facturozor_fields_settings_fn() {      
    global $wpdb;
    ?>
    <div class="wrap">                
      <h1><?php _e( 'Facturozor Fields Settings' ); ?></h1>
      <hr>
      <?php      
      if ( isset( $_REQUEST['wc'] ) ) {
        $facturozor = new WC_FACTUROZOR_MAIN();
        $modules = $facturozor->getModules();
        //var_dump($modules);
        $wc = sanitize_text_field($_REQUEST['wc']);//customers, products, orders
        if ( isset( $_REQUEST['submit'] ) ) {
          $wc_data = get_option( 'wc_facturozor_module_'.$wc );
          if ( ! is_array( $wc_data ) ) {
            $wc_data = array();
          }
          if($wc == 'customers' ){
            $wc_facturozor_module = 'Customers';
          }elseif($wc == 'orders' ){
            $wc_facturozor_module = 'Orders';
          }elseif($wc == 'products' ){
            $wc_facturozor_module = 'Products';
          }
          $wc_data['wc_facturozor_module'] = $wc_facturozor_module;
          $wc_data['wc_facturozor_fields'] = facturozor_recursive_sanitize_text_field($_REQUEST['wc_facturozor_fields']);
          $wc_data['module_saved'] = 1;
          
          update_option( 'wc_facturozor_module_'.$wc, $wc_data );
          ?>
            <div class="notice notice-success is-dismissible">
              <p><?php _e( 'Fields Mapping saved.' ); ?></p>
            </div>
          <?php
        } 
        
        $wc_data = get_option( 'wc_facturozor_module_'.$wc );
        //var_dump($wc_data);
        if ( ! is_array( $wc_data ) ) {
            $wc_data = array();
        }
        
        $wc_facturozor_module = ( isset( $wc_data['wc_facturozor_module'] ) ? $wc_data['wc_facturozor_module'] : '' );
        if($wc == 'customers' ){
          $wc_facturozor_module = 'Customers';
        }elseif($wc == 'orders' ){
          $wc_facturozor_module = 'Orders';
        }elseif($wc == 'products' ){
          $wc_facturozor_module = 'Products';
        }
        
        $wc_facturozor_fields = ( isset( $wc_data['wc_facturozor_fields'] ) ? $wc_data['wc_facturozor_fields'] : array() );
        ?>
        <h2><?php echo ( $wc == 'customers' ? __( 'Customers' ) : __( 'Orders' ) ); ?></h2>
        <hr>
        <form method="post">
          <table class="widefat striped">
            <thead>
              <tr>
                <th><?php echo ( $wc == 'customers' ? __( 'WooCommerce Field' ) : __( 'WooCommerce Field' ) ); ?></th>
                <th><?php _e( 'Field' ); ?></th>
              </tr>
            </thead>
            <tfoot>
              <tr>
                <th><?php echo ( $wc == 'customers' ? __( 'WooCommerce Field' ) : __( 'WooCommerce Field' ) ); ?></th>
                <th><?php _e( 'Field' ); ?></th>
              </tr>
            </tfoot>
            <tbody>
              <?php
                //$wc_facturozor_module == Customers Products Orders
                $modules_fields = get_option( 'wc_facturozor_modules_fields' );
                if ( isset( $modules_fields[$wc_facturozor_module] ) ) {
                  $fields = $modules_fields[$wc_facturozor_module];
                } else {
                  $fields = array();
                }
                
                if ( $wc == 'customers' ) {
                  $wc_fields = array(
                    'first_name'            => 'First Name',
                    'last_name'             => 'Last Name',
                    'user_email'            => 'Email',
                    'billing_company'       => 'Billing Company',
                    'billing_address_1'     => 'Billing Address 1',
                    'billing_city'          => 'Billing City',
                    'billing_state'         => 'Billing State',
                    'billing_postcode'      => 'Billing Postcode',
                    'billing_phone'         => 'Billing Phone',
                    'shipping_address_1'    => 'Shipping Address 1',
                    'shipping_city'         => 'Shipping City',
                    'shipping_postcode'     => 'Shipping Postcode',
                    'shipping_state'        => 'Shipping State',
                    'description'           => 'Biographical Info',
                    'display_name'          => 'Display name publicly as',
                  );
                    
                    
                } else if ( $wc == 'products' ) {
                  $wc_fields = array(
                    'name'=>'Name',
                    '_sku'=>'Ref No',
                    '_regular_price'              => 'Regular Price',
                    '_manage_stock'       => 'Manage Stock',
                    '_purchase_note'        => 'Description',
                    'post_date'                 => 'Date created',
                    'post_status'               => 'Status',
                  );
                    
                }
                else if ( $wc == 'orders' ) {
                  $wc_fields = array(
                    'order_number'              => 'Order Number',
                    '_payment_method_title'     => 'Payment Method Title',
                    '_cart_discount'            => 'Cart Discount',
                    '_cart_discount_tax'        => 'Cart Discount Tax',
                    '_order_shipping'           => 'Order Shipping',
                    '_order_shipping_tax'       => 'Order Shipping_tax',
                    '_order_tax'                => 'Order Tax',
                    '_order_total'              => 'Order Total',
                    'post_date'                 => 'Date created',
                    'post_status'               => 'Status',
                    'order_products'            => 'Order Products',
                  );
                    
                }
                
                foreach ( $wc_fields as $wc_field_key => $wc_field_value ) {
                  ?>
                  <tr>
                    <td><?php echo $wc_field_value; ?></td>
                    <td>
                      <select name="wc_facturozor_fields[<?php echo esc_attr($wc_field_key); ?>][key]">
                        <option value=""><?php _e( 'Select a field' ); ?></option>
                        <?php
                          if ( $fields != null ) {
                            foreach ( $fields as $field_key => $field_value ) {
                              $selected = '';
                              if( count($wc_facturozor_fields) > 0){
                                if ( isset( $wc_facturozor_fields[$wc_field_key]['key'] ) && $wc_facturozor_fields[$wc_field_key]['key'] == $field_key ) {
                                  $selected = ' selected="selected"';                                  
                                }  
                              }else{
                                //get from pre defined. 
                              }
                              
                              ?><option value="<?php echo esc_attr($field_key); ?>"<?php echo esc_attr($selected); ?>><?php echo esc_attr($field_value); ?></option><?php
                            }
                          }
                        ?>
                      </select>
                      <input type="hidden" name="wc_facturozor_fields[<?php echo esc_attr($wc_field_key); ?>][type]" value="text" />
                    </td>
                  </tr>
                  <?php
                }
              ?>
            </tbody>
          </table>
          <p>                              
            <input type='submit' class='button-primary' name="submit" value="<?php _e( 'Save Changes' ); ?>" />
          </p>
        </form>
        <?php
      } else {
        $facturozor = new WC_FACTUROZOR_MAIN();
        $modules = $facturozor->getModules();
        //var_dump($modules);
        //$modules = unserialize( get_option( 'wc_facturozor_modules' ) );                            
        $wc_facturozor_modules_fields = array();
        if ( $modules != null ) {
          $facturozor = new WC_FACTUROZOR_MAIN();
          foreach( $modules as $key => $value ) {
            $fields = $facturozor->getFieldsByModule( $key );
            $wc_facturozor_modules_fields[$key] = $fields;
          }
        }
        //var_dump($wc_facturozor_modules_fields);
        update_option( 'wc_facturozor_modules_fields', $wc_facturozor_modules_fields );
        ?>
        <table class="widefat striped">
          <thead>
            <tr>
              <th><?php _e( 'Module' ); ?></th>
              <th><?php _e( 'Mapped' ); ?></th>       
              <th><?php _e( 'Action' ); ?></th>
            </tr>
          </thead>
          <tfoot>
            <tr>
              <th><?php _e( 'Module' ); ?></th>
              <th><?php _e( 'Mapped' ); ?></th>       
              <th><?php _e( 'Action' ); ?></th>
            </tr>
          </tfoot>
          <tbody>
            <tr>
              <td><?php _e( 'Customers' ); ?></td>
              <td>
                <?php
                  $wc_data = get_option( 'wc_facturozor_module_customers' );
                  if ( ! is_array( $wc_data ) ) {
                      $wc_data = array();
                  }
                  
                  echo ( isset( $wc_data['module_saved'] ) && $wc_data['module_saved'] == 1 ? '<span class="dashicons dashicons-yes"></span>' : '<span class="dashicons dashicons-no"></span>' );
                ?>
              </td>
              <td><a href="<?php echo menu_page_url( 'wc_facturozor_fields_settings', 0 ); ?>&wc=customers"><span class="dashicons dashicons-edit"></span></a></td>
            </tr>
            <tr>
              <td><?php _e( 'Products' ); ?></td>
              <td>
                <?php
                  $wc_data = get_option( 'wc_facturozor_module_products' );
                  if ( ! is_array( $wc_data ) ) {
                      $wc_data = array();
                  }
                  
                  echo ( isset( $wc_data['module_saved'] ) && $wc_data['module_saved'] == 1 ? '<span class="dashicons dashicons-yes"></span>' : '<span class="dashicons dashicons-no"></span>' );
                ?>
              </td>
              <td><a href="<?php echo menu_page_url( 'wc_facturozor_fields_settings', 0 ); ?>&wc=products"><span class="dashicons dashicons-edit"></span></a></td>
            </tr>
            <tr>
              <td><?php _e( 'Orders' ); ?></td>
              <td>
                <?php
                  $wc_data = get_option( 'wc_facturozor_module_orders' );
                  if ( ! is_array( $wc_data ) ) {
                      $wc_data = array();
                  }
                  
                  echo ( isset( $wc_data['module_saved'] ) && $wc_data['module_saved'] == 1 ? '<span class="dashicons dashicons-yes"></span>' : '<span class="dashicons dashicons-no"></span>' );
                ?>
              </td>
              <td><a href="<?php echo menu_page_url( 'wc_facturozor_fields_settings', 0 ); ?>&wc=orders"><span class="dashicons dashicons-edit"></span></a></td>
            </tr>
          </tbody>
        </table>
        <?php
      }
      ?>
    </div>
    <?php
  }
}

/*
 * synch all.
 * * For sync . can be done with a simple call
 * show last sync time.
 * show last order, product, customer sync
 */
if ( ! function_exists( 'wc_facturozor_sync_fn' ) ) {
  function wc_facturozor_sync_fn() {
      
    $wc_module_facturozor = ( isset( $_REQUEST['wc_module_facturozor'] ) ? sanitize_text_field($_REQUEST['wc_module_facturozor']) : '' );
    if ( isset( $_REQUEST['submit'] ) ) {
      if ( $wc_module_facturozor == 'customers' ) {
        wc_facturozor_customers_synch_all();                    
        ?>
        <div class="notice notice-success is-dismissible">
          <p><?php _e( 'Customers Synched successfully.' ); ?></p>
        </div>
        <?php
      }elseif ( $wc_module_facturozor == 'products' ) { 
        wc_facturozor_products_synch_all();                    
        ?>
        <div class="notice notice-success is-dismissible">
          <p><?php _e( 'Prodcuts Synched successfully.' ); ?></p>
        </div>
        <?php
      }elseif ( $wc_module_facturozor == 'orders' ) { 
        wc_facturozor_orders_synch_all();                    
        ?>
        <div class="notice notice-success is-dismissible">
          <p><?php _e( 'Orders Synched successfully.' ); ?></p>
        </div>
        <?php
      }else{
        ?>
        <div class="notice notice-success is-dismissible">
          <p><?php _e( 'Please select Type.' ); ?></p>
        </div>
        <?php
      }
    }
    ?>
    <div class="wrap">                
      <h1><?php _e( 'Facturozor Synch' ); ?></h1>
      <hr>
      <form method="post" action="<?php echo menu_page_url( 'wc_facturozor_sync', 0 ); ?>">
        <table class="form-table">
          <tbody>
            <tr>
              <th scope="row"><?php _e( 'Module' ); ?></th>
              <td>
                <select name="wc_module_facturozor" required="required">
                  <option value=""><?php _e( 'Please select Module to Synch Manually' ); ?></option>
                  <?php
                    $options = array(
                      'customers' => __( 'Customers' ),
                      'prodcuts' => __( 'Products' ),
                      'orders'    => __( 'Orders' ),
                    );
                    foreach ( $options as $key => $value ) {
                      $selected = '';
                      if ( $wc_module_facturozor == $key ) {
                          $selected = ' selected="selected"';
                      }
                      ?>
                      <option value="<?php echo esc_attr($key); ?>"<?php echo $selected; ?>><?php echo esc_attr($value); ?></option><?php
                    }
                  ?>
                </select>
              </td>
            </tr>                        
          </tbody>
        </table>
          <p class="submit">
            <input type="submit" name="submit" class="button button-primary" value="<?php _e( 'Synch Now' ); ?>" />
          </p>
      </form>           
    </div>
    <?php
  }
}