<?php
if ( ! defined( 'ABSPATH' ) ) {
    exit( 'restricted access' );
}

/*
 * This is a class for Zoho CRM API.
 */
if ( ! class_exists( 'WC_FACTUROZOR_MAIN' ) ) {
    class WC_FACTUROZOR_MAIN {
      var $synhModuleBaseUrl;
      
      function __construct() {
        $wc_facturozor_api_key = get_option( 'wc_facturozor_api_key' ); 
        $site_url  = site_url();
        $this->$synhModuleBaseUrl = "https://www.facturozor.app/v1/api/";
        $this->wc_facturozor_api_key = $wc_facturozor_api_key;
        $this->site_url = $site_url;
      }   
      
      function getModules(){
        $modules = array(
          'Customers'=>'Customers',
          'Products' =>'Products',
          'Orders' =>'Orders',
        );
        return $modules;
      }
      
      function getFieldsByModule($module){
        //a:2:{s:9:"Customers";s:9:"Customers";s:8:"Products";s:8:"Products";;s:6:"Orders";s:6:"Orders";}' );
        $fields = array();
        if($module == 'Customers'){
          $fields = array(
            'name'          => 'Prénom Nom',
            'first_name'            => 'Prénom',
            'last_name'             => 'Nom',
            'email'            => 'Email',
            'name_of_enterprise'       => 'Nom de l\'entreprise',
            'address'     => 'Adresse',
            'city'          => 'Ville',
            'state'         => 'Région',
            'zip'      => 'Code Postal',
            'phone'         => 'Tel portable',
            'delivery_address'    => 'Adresse de livraison',
            'delivery_city'         => 'Ville',
            'delivery_zip'     => 'Code Postal',
            'delivery_state'        => 'Région',
            'note'           => 'Note',
          );
        }
        elseif ($module == 'Orders') {
          $fields = array(
            'order_number'              => 'Order Number',
            '_payment_method_title'     => 'Payment Method Title',
            '_cart_discount'            => 'Cart Discount',
            '_cart_discount_tax'        => 'Cart Discount Tax',
            '_order_shipping'           => 'Order Shipping',
            '_order_shipping_tax'       => 'Order Shipping_tax',
            '_order_tax'                => 'Order Tax',
            '_order_total'              => 'Order Total',
            'post_date'                 => 'Date created',
            'post_status'               => 'Status',
            'order_products'            => 'Order Products',
          );
        }
        elseif ($module == 'Products') {
          $fields = array(
            'name'=>'Name',
            '_sku'=>'Ref No',
            '_regular_price'              => 'Regular Price',
            '_manage_stock'       => 'Manage Stock',
            '_purchase_note'        => 'Description',
            'post_date'                 => 'Date created',
            'post_status'               => 'Status',
          );
        }
        return $fields;
      }
      
      
      function addRecord( $module, $data ) {            
        $url = $this->$synhModuleBaseUrl.''.$module.'/synch_create';
        $dataObj = array(
          'api_key' => $this->wc_facturozor_api_key,
          'site_url'  => $this->site_url,
          'data' => json_encode( $data ),            
        );
        $options = [
          'body'        => wp_json_encode( $dataObj ),
          'headers'     => [
              'Content-Type' => 'application/json',
          ],          
        ];
        $apiResponse = wp_remote_post( $url, $options );
        $response = json_decode( wp_remote_retrieve_body( $apiResponse ) );
        //if ( isset($response) && isset( $response['status'] ) && $response['status'] == 'error' ) {
        if ( isset($response) && isset( $response->status ) && $response->status == 'error' ) {  
          $log = "Error Create for Module : ".$module."\n";
          $log .= "Msg: ".$response->msg."\n";
          $log .= "Date: ".date( 'Y-m-d H:i:s' )."\n\n";          
        }
        return $response;
      }
      
      function updateRecord( $module, $data, $id ) {
        $url = $this->$synhModuleBaseUrl.''.$module.'/synch_update';
        $dataObj = array(
          'api_key' => $this->wc_facturozor_api_key,
          'site_url'  => $this->site_url,
          'data' => json_encode( $data ),      
          'id' => $id
        );
        $options = [
          'body'        => wp_json_encode( $dataObj ),
          'headers'     => [
              'Content-Type' => 'application/json',
          ],          
        ];        
        $apiResponse = wp_remote_post( $url, $options );
        $response = json_decode( wp_remote_retrieve_body( $apiResponse ) );
        if ( isset($response) && isset( $response->status ) && $response->status == 'error' ) {  
          $log = "Error Update for Module : ".$module." ID :".$id ."\n";
          $log .= "Msg: ".$response->msg."\n";
          $log .= "Date: ".date( 'Y-m-d H:i:s' )."\n\n";
        }
        return $response;
      }

      function updatePayment($data){
        $url = $this->$synhModuleBaseUrl.'/synch_update_payment';
        $dataObj = array(
          'api_key' => $this->wc_facturozor_api_key,
          'site_url'  => $this->site_url,
          'data' => json_encode( $data ),                
        );
        $options = [
          'body'        => wp_json_encode( $dataObj ),
          'headers'     => [
              'Content-Type' => 'application/json',
          ],          
        ];
        $apiResponse = wp_remote_post( $url, $options );
        $response = json_decode( wp_remote_retrieve_body( $apiResponse ) );
        if ( isset($response) && isset( $response->status ) && $response->status == 'error' ) {  
          $log = "Error Update for payment of order ID :".$data['id'] ."\n";
          $log .= "Msg: ".$response->msg."\n";
          $log .= "Date: ".date( 'Y-m-d H:i:s' )."\n\n";
        }
        return $response;
      }
    }
}